<?php
/*
Plugin Name: DynamicSitemap
Plugin URI: http://hubertsworld.prince.homedns.org/
Description: Generates a dynamic sitemap
Author: Don Prince
Version: 1.0
Author URI: http://hubertsworld.prince.homedns.org/

Usage
* The categories are comma seperated.
* Unless supplied with a list of catergories, the 'include' attribute will default to ALL categories.
* The 'exclude' attribute defaults to exclude NONE.
* This is simple, you are able to pointlessly include and exclude the same category.
* The is NO closing tag.

Example:
[dynamicsitemap include="cat 1,cat2, cat 3" exclude="cat 5, cat 6, cat7"]
*/

if (!function_exists('strpos_array')) { // found on php.net (I think :))
    function strpos_array($haystack, $needles) {
        if ( is_array($needles) ) {
            foreach ($needles as $str) {
                if ( is_array($str) ) {
                    $pos = strpos_array($haystack, $str);
                } else {
$pos = strpos($haystack, $str);
                }
                if ($pos !== false) {
                    return $pos;
                }
            }
            return false;
        } else {
            return strpos($haystack, $needles);
        }
    }
}

add_filter('init', 'plugin_dynamicsitemap_tags');

function plugin_dynamicsitemap_tags() {
        $bbcode =& plugin_bbcode_init(); //import the "global" bbcode object into current function
                                         // this way 
                                         // a) parsing is done only once, and by the official plugin
                                         // b) you create only ONE object, and therefore computation is quicker
        $bbcode->addCode (
                    'dynamicsitemap',  // tag name: this will go between square brackets
                    'callback_replace_single', // type of action: we'll use a callback function
                    'plugin_dynamicsitemap_callback', // name of the callback function
                    array('usecontent_param' => array ('include', 'exclude')), // supported parameters
                    'block', // type of the tag, inline or block, etc
                    array ('listitem', 'block','inline'), // type of elements in which you can use this tag
                    array ()); // type of elements where this tag CAN'T go (in this case, none, so it can go everywhere)
 
        $bbcode->setCodeFlag ('dynamicsitemap', 'closetag', BBCODE_CLOSETAG_FORBIDDEN); 
$bbcode->setCodeFlag ('dynamicsitemap', 'opentag.before.newline', BBCODE_NEWLINE_DROP); 
$bbcode->setCodeFlag ('dynamicsitemap', 'opentag.after.newline', BBCODE_NEWLINE_DROP); 
$bbcode->setCodeFlag ('dynamicsitemap', 'closetag.before.newline', BBCODE_NEWLINE_DROP); 
$bbcode->setCodeFlag ('dynamicsitemap', 'closetag.after.newline', BBCODE_NEWLINE_DROP); 
}


function plugin_dynamicsitemap_callback($action, $attributes, $content, $params, &$node_object) { 
    if ($action == 'validate') {
        // not used for now
        return true;
    }

    global $fpdb;
    global $post;
    $current_post = $post;

    /****************** This part was suggested by NoWhereMan  *************************/
    $q = new FPDB_Query(array('start'=>0, 'count'=>-1, 'fullparse'=>1), 0);
    while($q->hasmore()) {
        list($id, $entry) = $q->getEntry();
        $ds_link = get_permalink($id);
        foreach ($entry['categories'] as $catid) {
$entlist[$catid][$id] = array(
'link' => $ds_link,
'subject' => $entry['subject']
);
}
        // $entry['subject'] etc etc;
        // notice that with fullparse = 0 you'll get only subject, date, id;
        // set it to =1 if you want categories and author as well
    }
    $post= $current_post;

// Get categories 'cat id'=>'cat title'
    $catlist = $fpdb->_categories['defs'];

// check for inclusions/exclusions
$inc = array();
$exc = array();
if (isset($attributes['include']) && !empty($attributes['include'])){
    $ds_tmpstr = str_replace('&amp;', chr(38), $attributes['include']);
    $inc = explode(",",$ds_tmpstr);
} else {
foreach ($catlist as $catid => $cattitle)
$inc[] = $cattitle;
}
if (isset($attributes['exclude']) && !empty($attributes['exclude'])){
    $ds_tmpstr = str_replace('&amp;', chr(38), $attributes['exclude']);
    $exc = explode(",",$ds_tmpstr);
}

    // output site list
    $ds_tmpstr = "<div class=\"sitemap\">";
    foreach ($catlist as $catid => $cattitle) {
        if ((strpos_array($cattitle, $inc) !== false) &&
(strpos_array($cattitle, $exc) === false) &&
(isset($entlist[$catid]))){
            $ds_tmpstr .= "<p class=\"sitemap\"><b>$cattitle</b></p>";
            $ds_tmpstr .= "<ul class=\"sitemap\">";
            foreach ($entlist as $entcatid => $catents ) {
                if ( $entcatid == $catid ) {
                    foreach ($catents as $entid => $ent ) {
                        $ds_tmpstr .= "<li class=\"sitemap\"><a href=\"" . $ent['link'] . "\">" . $ent['subject'] . "</a></li>";
                    }
                }
            }
$ds_tmpstr .= "</ul>";
}
}
    $ds_tmpstr .= "</div>";

/***********************************************************************************/

return $ds_tmpstr;
}

?>
